==> soxspipe-0.5.0.tar.gz: soxspipe-0.5.0/soxspipe/commonutils/dispersion_map_to_pixel_arrays.py <==
#!/usr/bin/env python
# encoding: utf-8
"""
*use a first-guess dispersion map to convert wavelengths to pixels*

:Author:
    David Young

:Date Created:
    April 15, 2021
"""
from builtins import object
import sys
import os
os.environ['TERM'] = 'vt100'
from fundamentals import tools
from os.path import expanduser
import unicodecsv as csv
from soxspipe.commonutils.polynomials import chebyshev_order_wavelength_polynomials
import numpy as np


def dispersion_map_to_pixel_arrays(
        log,
        dispersionMapPath,
        orderPixelTable):
    """*use a first-guess dispersion map to append x,y fits to line-list data frame.* 

    Return a line-list with x,y fits given a first guess dispersion map.*

    **Key Arguments:**

    - `log` -- logger
    - `dispersionMapPath` -- path to the dispersion map
    - `orderPixelTable` -- a data-frame including 'order', 'wavelength' and 'slit_pos' columns

    **Usage:**

    ```python
    from soxspipe.commonutils import dispersion_map_to_pixel_arrays
    myDict = {
        "order": [11, 11, 11],
        "wavelength": [850.3, 894.3, 983.2],
        "slit_position": [0, 0, 0]
    }
    orderPixelTable = pd.DataFrame(myDict)
    orderPixelTable = dispersion_map_to_pixel_arrays(
        log=log,
        dispersionMapPath="/path/to/map.csv",
        orderPixelTable=orderPixelTable
    )
    ```           
    """
    log.debug('starting the ``dispersion_map_to_pixel_arrays`` function')

    # READ THE FILE
    home = expanduser("~")
    dispersion_map = dispersionMapPath.replace("~", home)

    # READ IN THE X- AND Y- GENERATING POLYNOMIALS FROM DISPERSION MAP FILE
    coeff = {}
    poly = {}
    with open(dispersion_map, 'rb') as csvFile:
        csvReader = csv.DictReader(
            csvFile, dialect='excel', delimiter=',', quotechar='"')
        for row in csvReader:
            axis = row["axis"]
            order_deg = int(row["order-deg"])
            wavelength_deg = int(row["wavelength-deg"])
            slit_deg = int(row["slit-deg"])
            coeff[axis] = [float(v) for k, v in row.items() if k not in [
                "axis", "order-deg", "wavelength-deg", "slit-deg"]]
            poly[axis] = chebyshev_order_wavelength_polynomials(
                log=log, order_deg=order_deg, wavelength_deg=wavelength_deg, slit_deg=slit_deg).poly
    csvFile.close()

    # CONVERT THE ORDER-SORTED WAVELENGTH ARRAYS INTO ARRAYS OF PIXEL TUPLES
    orderPixelTable["fit_x"] = poly['x'](orderPixelTable, *coeff['x'])
    orderPixelTable["fit_y"] = poly['y'](orderPixelTable, *coeff['y'])

    # FILTER DATA FRAME
    # FIRST CREATE THE MASK
    mask = (orderPixelTable["fit_x"] > 0) & (orderPixelTable["fit_y"] > 0)
    orderPixelTable = orderPixelTable.loc[mask]

    log.debug('completed the ``dispersion_map_to_pixel_arrays`` function')
    return orderPixelTable

==> soxspipe-0.6.2.tar.gz: soxspipe-0.6.2/soxspipe/commonutils/dispersion_map_to_pixel_arrays.py <==
#!/usr/bin/env python
# encoding: utf-8
"""
*use a first-guess dispersion map to convert wavelengths to pixels*

:Author:
    David Young

:Date Created:
    April 15, 2021
"""
import numpy as np
from soxspipe.commonutils.polynomials import chebyshev_order_wavelength_polynomials
import unicodecsv as csv
from os.path import expanduser
from fundamentals import tools
from builtins import object
import sys
import os
from astropy.table import Table
os.environ['TERM'] = 'vt100'


def dispersion_map_to_pixel_arrays(
        log,
        dispersionMapPath,
        orderPixelTable):
    """*use a first-guess dispersion map to append x,y fits to line-list data frame.* 

    Return a line-list with x,y fits given a first guess dispersion map.*

    **Key Arguments:**

    - `log` -- logger
    - `dispersionMapPath` -- path to the dispersion map
    - `orderPixelTable` -- a data-frame including 'order', 'wavelength' and 'slit_pos' columns

    **Usage:**

    ```python
    from soxspipe.commonutils import dispersion_map_to_pixel_arrays
    myDict = {
        "order": [11, 11, 11],
        "wavelength": [850.3, 894.3, 983.2],
        "slit_position": [0, 0, 0]
    }
    orderPixelTable = pd.DataFrame(myDict)
    orderPixelTable = dispersion_map_to_pixel_arrays(
        log=log,
        dispersionMapPath="/path/to/map.csv",
        orderPixelTable=orderPixelTable
    )
    ```           
    """
    log.debug('starting the ``dispersion_map_to_pixel_arrays`` function')

    # READ THE FILE
    home = expanduser("~")
    dispersion_map = dispersionMapPath.replace("~", home)

    # SPEC FORMAT TO PANDAS DATAFRAME
    dat = Table.read(dispersion_map, format='fits')
    tableData = dat.to_pandas()

    # READ IN THE X- AND Y- GENERATING POLYNOMIALS FROM DISPERSION MAP FILE
    coeff = {}
    poly = {}
    check = 1
    for index, row in tableData.iterrows():
        axis = row["axis"].decode("utf-8")
        order_deg = int(row["order-deg"])
        wavelength_deg = int(row["wavelength-deg"])
        slit_deg = int(row["slit-deg"])

        if check:
            for i in range(0, order_deg + 1):
                orderPixelTable[f"order_pow_{i}"] = orderPixelTable["order"].pow(i)
            for j in range(0, wavelength_deg + 1):
                orderPixelTable[f"wavelength_pow_{j}"] = orderPixelTable["wavelength"].pow(j)
            for k in range(0, slit_deg + 1):
                orderPixelTable[f"slit_position_pow_{k}"] = orderPixelTable["slit_position"].pow(k)
            check = 0

        coeff[axis] = [float(v) for k, v in row.items() if k not in [
            "axis", "order-deg", "wavelength-deg", "slit-deg"]]
        poly[axis] = chebyshev_order_wavelength_polynomials(
            log=log, order_deg=order_deg, wavelength_deg=wavelength_deg, slit_deg=slit_deg, exponents_included=True).poly

    # CONVERT THE ORDER-SORTED WAVELENGTH ARRAYS INTO ARRAYS OF PIXEL TUPLES
    orderPixelTable["fit_x"] = poly['x'](orderPixelTable, *coeff['x'])
    orderPixelTable["fit_y"] = poly['y'](orderPixelTable, *coeff['y'])

    # FILTER DATA FRAME
    # FIRST CREATE THE MASK
    mask = (orderPixelTable["fit_x"] > 0) & (orderPixelTable["fit_y"] > 0)
    orderPixelTable = orderPixelTable.loc[mask]

    log.debug('completed the ``dispersion_map_to_pixel_arrays`` function')
    return orderPixelTable
